#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "rts/rts.h"


int
main(int argc, char *argv[])
{
    RTS *rp;
    int code, i;
    time_t o_t, c_t;
    int do_flush = 0;
    int drop_drive = 0;
    int dump_event_log = 0;
    int test_fail = 0;
    int set_time = 0;
    int show_ssi = 0;
    int show_dsi = 0;
    int show_ddi = 0;
    int show_rgi = -1;
    int show_hp = -1;
    
	
    if (argc < 2)
    {
	fprintf(stderr, "Usage: %s <device>\n", argv[0]);
	exit(1);
    }

    for (i = 1; i < argc && argv[i][0] == '-'; i++)
    {
	if (strcmp(argv[i], "-flush") == 0)
	    do_flush = 1;
	else if (strcmp(argv[i], "-debug") == 0)
	    rts_set_debug(1);
	else if (strcmp(argv[i], "-drop_drive") == 0)
	    drop_drive = 1;
	else if (strcmp(argv[i], "-show_hp") == 0)
	{
	    if (i+1 < argc && isdigit((int) (argv[i+1][0])))
		show_hp = atoi(argv[++i]);
	    else
	    {
		fprintf(stderr, "Missing host channel\n");
		exit(1);
	    }
	}
	else if (strcmp(argv[i], "-test_fail") == 0)
	    test_fail = 1;
	else if (strcmp(argv[i], "-set_time") == 0)
	    set_time = 1;
	else if (strcmp(argv[i], "-show_ssi") == 0)
	    show_ssi = 1;
	else if (strcmp(argv[i], "-show_dsi") == 0)
	    show_dsi = 1;
	else if (strcmp(argv[i], "-show_ddi") == 0)
	    show_ddi = 1;
	else if (strcmp(argv[i], "-show_rgi") == 0)
	{
	    if (i+1 < argc && isdigit((int) (argv[i+1][0])))
		show_rgi = atoi(argv[++i]);
	    else
	    {
		fprintf(stderr, "Missing redundancy group number\n");
		exit(1);
	    }
	}
	else if (strcmp(argv[i], "-dump_events") == 0)
	    dump_event_log = 1;
	else
	{
	    fprintf(stderr, "Unknown command line switch: %s\n", argv[i]);
	    exit(1);
	}
    }
    

    rp = rts_open(argv[i]);
    if (rp == NULL)
    {
	perror("rts_open");
	exit(1);
    }

    if (do_flush)
    {
	puts("Flushing old RTS messages (this may take some time)");
	rts_flush_messages(rp);
    }

    if (set_time)
    {
	puts("READ DATE AND TIME");
	code = rts_read_date_and_time(rp, &c_t);
	if (code)
	{
	    rts_perror(stderr, "rts_read_date_and_time", rp);
	    exit(1);
	}
	
	printf("Time on controller: %s\n", ctime(&c_t));

	time(&o_t);
	if (fabs(difftime(o_t, c_t)) > 5)
	{
	    puts("BEGIN SESSION");
	    code = rts_begin_session(rp, 0);
	    if (code)
	    {
		rts_perror(stderr, "rts_begin_session", rp);
		exit(1);
	    }
	    
	    puts("WRITE DATE AND TIME");
	    code = rts_write_date_and_time(rp, o_t);
	    if (code)
	    {
		rts_perror(stderr, "rts_write_date_and_time", rp);
		rts_end_session(rp);
		exit(1);
	    }
	    
	    puts("END SESSION");
	    code = rts_end_session(rp);
	    if (code)
	    {
		rts_perror(stderr, "rts_end_session", rp);
		exit(1);
	    }
	}
    }

    if (test_fail)
    {
	puts("\nDROP DRIVE (should fail)");
	code = rts_drop_drive(rp, 10, 1, 5);
	if (code)
	{
	    rts_perror(stderr, "rts_drop_drive", rp);
	    rts_end_session(rp);
	    exit(1);
	}
    }
    
    if (drop_drive)
    {
	puts("BEGIN SESSION");
	code = rts_begin_session(rp, 0);
	if (code)
	{
	    rts_perror(stderr, "rts_begin_session", rp);
	    exit(1);
	}
	
	puts("\nDROP DRIVE (should fail)");
	code = rts_drop_drive(rp, 10, 1, 5);
	if (code)
	{
	    rts_perror(stderr, "rts_drop_drive", rp);
	    rts_end_session(rp);
	    exit(1);
	}
	    
	puts("END SESSION");
	code = rts_end_session(rp);
	if (code)
	{
	    rts_perror(stderr, "rts_end_session", rp);
	    exit(1);
	}
    }

    if (show_hp >= 0)
    {
	struct rts_host_parameters hp;
	
	puts("READ HOST PARAMETERS");
	code = rts_read_host_parameters(rp, show_hp, &hp);
	if (code)
	{
	    rts_perror(stderr, "rts_read_host_parameters", rp);
	    exit(1);
	}

	rts_print_host_parameters(stdout, &hp);
    }

    
    if (show_ssi)
    {
	struct rts_static_system_information ssi;

	puts("READ STATIC SYSTEM INFORMATION");
	code = rts_read_static_system_information(rp, &ssi);
	if (code)
	{
	    rts_perror(stderr, "rts_read_static_system_information", rp);
	    exit(1);
	}

	printf("Firmware Version: %.9s\n", ssi.firmware_version);
	printf("RTS Version: %d\n", ssi.rts_version);
	printf("Cache size: %d MB\n", rts_w2i(ssi.cache_size));
    }

    if (show_dsi)
    {
	struct rts_dynamic_system_information dsi;

	
	puts("\nREAD DYNAMIC SYSTEM INFORMATION");
	code = rts_read_dynamic_system_information(rp, &dsi);
	if (code)
	{
	    rts_perror(stderr, "rts_read_dynamic_system_information", rp);
	    exit(1);
	}

	printf("Online redundancy groups: %x\n",
	       rts_q2i(dsi.online_redundancy_groups));
	printf("Remote access level: %d\n", dsi.remote_access_level);
    }

    if (show_ddi)
    {
	struct rts_disk_drive_information ddi;

	puts("\nDISK DRIVE INFORMATION");
	code = rts_read_disk_drive_information(rp, 8, 3, &ddi);
	if (code)
	{
	    rts_perror(stderr, "rts_read_disk_drive_information", rp);
	    exit(1);
	}

	printf("Status: %d\n", ddi.status);
	printf("RAID Set: %d\n", ddi.raid_set);
	printf("Member: %d\n", ddi.member);
	printf("Capacity: %d\n", rts_q2i(ddi.capacity));
	printf("Disk Vendor: %.8s\n", ddi.disk_vendor);
	printf("Disk Model: %.16s\n", ddi.disk_model);
	printf("Disk Revision: %.4s\n", ddi.disk_revision);
    }

    if (show_rgi >= 0)
    {
	struct rts_redundancy_group_information rb;
	

	puts("\nREDUNDANCY GROUP INFORMATION");
	code = rts_read_redundancy_group_information(rp, show_rgi, &rb);
	if (code)
	{
	    rts_perror(stderr, "rts_read_redundancy_group_information", rp);
	    exit(1);
	}

	rts_print_redundancy_group_information(stdout, &rb);
    }

    if (dump_event_log)
    {
	struct rts_event_log_information elb;
	struct rts_event_information eb;
	int i, first, last;
	time_t bt;
	
	
	puts("\nEVENT LOG INFORMATION\n");
	code = rts_read_event_log_information(rp, &elb);
	if (code)
	{
	    rts_perror(stderr, "rts_read_event_log_information", rp);
	    exit(1);
	}

	if (elb.status)
	{
	    first = rts_w2i(elb.oldest_event_sequence_number);
	    last = rts_w2i(elb.newest_event_sequence_number);

	    for (i = first; i <= last; i++)
	    {
		code = rts_read_event_information(rp, i, &eb);
		if (code)
		{
		    rts_perror(stderr, "rts_read_event_information", rp);
		    exit(1);
		}
		
		bt = rts_mktime(&eb.time_stamp);
#if 0
		printf("%5d: version=%3d type=%3d time=%s",
		       rts_w2i(eb.sequence_number),
		       eb.event_log_version,
		       eb.event_type,
		       ctime(&bt));
#endif
		puts("------------------------------------------------------------------------------");
		rts_print_event(stdout, &eb);
	    }
	}
    }
	
	
    return 0;
}
