/*
** rts/command.h
**
** Viper RTS Command functions
**
** Copyright (c) 1998 Peter Eriksson <pen@lysator.liu.se>
*/

#ifndef VIPER_RTS_COMMAND_H
#define VIPER_RTS_COMMAND_H

/* RTS Command Messages */
#define RTS_REBUILD_DISK_DRIVE			0
#define RTS_CREATE_RAID_SET			1
#define RTS_ADD_SPARE				2
#define RTS_CHANGE_PASSWORD			3
#define RTS_SCAN_DRIVE				4
#define RTS_FLUSH_EVENT_LOG			7
#define RTS_BEGIN_SHUTDOWN			8
#define RTS_COMPLETE_SHUTDOWN			9
#define RTS_DROP_DRIVE				10
#define RTS_BEGIN_SESSION			11
#define RTS_END_SESSION				12
#define RTS_FORMAT_DRIVE			13
#define RTS_DELETE_RAID_SET			17
#define RTS_SET_ALARM_STATE			18
#define RTS_SWITCHOVER_CONTROLLER		19



/*
** RTS Command Message Structures
*/

/* 4.1 Rebuild Disk Drive */
struct rts_rebuild_disk_drive
{
    Byte timeout;
    Byte raid_set;
    Byte channel;
    Byte scsi_id;
};

/* 4.2 Create RAID Set */
struct rts_create_raid_set
{
    Byte timeout;
    Byte raid_set;
    Byte raid_level;
    Byte nrgroups;
    Word c0_drive_sel;
    Word c1_drive_sel;
    Word c2_drive_sel;
    Word c3_drive_sel;
    Word c4_drive_sel;
    Word c5_drive_sel;
    Word c6_drive_sel;
    Word c7_drive_sel;
    Word c8_drive_sel;
    Word c9_drive_sel;
    
    struct
    {
	Word redundancy_group;
	Word chunk_size;
	Quad capacity;
    } rgd[1]; /* Can be multiple */
};

/* 4.3 Add Spare */
struct rts_add_spare
{
    Byte timeout;
    Byte spare_designation;
    Byte channel;
    Byte scsi_id;
};

/* 4.4 Change Password */
struct rts_change_password
{
    Byte password_type;
    Byte new_password[9];
};

/* 4.5 Scan Drive */
struct rts_scan_drive
{
    Byte timeout;
    Byte channel;
    Byte scsi_id;
};

/* 4.8 Complete Shutdown */
struct rts_complete_shutdown
{
    Byte final_shutdown_action;
};

/* 4.9 Drop Drive */
struct rts_drop_drive
{
    Byte timeout;
    Byte channel;
    Byte scsi_id;
};

/* 4.10 Begin Session */
struct rts_begin_session
{
    Byte user_information[64];
    Byte system_information[64];
    Byte session_type;
};

/* 4.12 Format Drive */
struct rts_format_drive
{
    Byte timeout;
    Byte channel;
    Byte scsi_id;
};

/* 4.13 Delete RAID Set */
struct rts_delete_raid_set
{
    Byte timeout;
    Byte raid_set;
};

/* 4.14 Set Alarm State */
struct rts_set_alarm_state
{
    Byte alarm_state;
};


union rts_command_message
{
    struct rts_rebuild_disk_drive 	rebuild_disk_drive;
    struct rts_create_raid_set 		create_raid_set;
    struct rts_add_spare		add_spare;
    struct rts_change_password		change_password;
    struct rts_scan_drive		scan_drive;
    struct rts_complete_shutdown	complete_shutdown;
    struct rts_drop_drive		drop_drive;
    struct rts_begin_session		begin_session;
    struct rts_format_drive		format_drive;
    struct rts_delete_raid_set		delete_raid_set;
    struct rts_set_alarm_state		set_alarm_state;
};



extern int
rts_rebuild_disk_drive(RTS *rp,
		       int timeout,
		       int raid_set,
		       int channel,
		       int scsi_id);

extern int
rts_create_raid_set(RTS *rp,
		    struct rts_create_raid_set *crsp);

extern int
rts_add_spare(RTS *rp,
	      int timeout,
	      int spare_designation,
	      int channel,
	      int scsi_id);

extern int
rts_change_password(RTS *rp,
		    int password_type,
		    const char *new_password);

extern int
rts_scan_drive(RTS *rp,
	       int timeout,
	       int channel,
	       int scsi_id);

extern int
rts_flush_event_log(RTS *rp);

extern int
rts_begin_shutdown(RTS *rp);

extern int
rts_complete_shutdown(RTS *rp,
		      int final_shutdown_action);

extern int
rts_drop_drive(RTS *rp,
	       int timeout,
	       int channel,
	       int scsi_id);

extern int
rts_begin_session(RTS *rp,
		  int session_type);

extern int
rts_end_session(RTS *rp);

extern int
rts_set_alarm_state(RTS *rp,
		    int alarm_state);

#endif
