/*
** rts_event.h
**
** Structure and definitions for Viper RTS Event log
*/

#ifndef VIPER_RTS_EVENT_H
#define VIPER_RTS_EVENT_H


/* RTS Event Types */
#define RTS_UNKNOWN_EVENT					 0
#define RTS_SCSI_EVENT						 1
#define RTS_PORT_EVENT						 2
#define RTS_PARITY_ERROR_EVENT					 3
#define RTS_CREATE_STARTED_EVENT				 4
#define RTS_REBUILD_STARTED_EVENT				 5
#define RTS_CREATE_COMPLETED_EVENT				 6
#define RTS_REBUILD_COMPLETED_EVENT				 7
#define RTS_DROPPED_DRIVE_EVENT					 8
#define RTS_TEMPERATURE_CHANGED_EVENT				 9
#define RTS_BAD_FAN_EVENT					10
#define RTS_DRIVE_SEEN_EVENT					11
#define RTS_DRIVE_GONE_EVENT					12
#define RTS_VALIDATION_CONFLICT_EVENT				13
#define RTS_SCSI_HOST_EVENT					14
#define RTS_REASSIGN_BLOCK_EVENT				15
#define RTS_INTERNAL_ENVIRONMENTAL_SYSTEM_EVENT			16
#define RTS_PORT_SCSI_SYNCHRONOUS_NEGOTIATION_EVENT		17
#define RTS_PORT_SCSI_WIDE_DMA_NEGOTIATION_EVENT		18
#define RTS_HOST_SCSI_SYNCHRONOUS_NEGOTIATION_EVENT		19
#define RTS_HOST_SCSI_WIDE_DMA_NEGOTIATION_EVENT		20
#define RTS_FORCED_ERROR_WRITTEN_EVENT				21
#define RTS_UNEXPECTED_PORT_INTERRUPT_EVENT			22
#define RTS_ADD_SPARE_STARTED_EVENT				23
#define RTS_ADD_SPARE_COMPLETED_EVENT				24
#define RTS_SYSTEM_POWER_UP_EVENT				25
#define RTS_SYSTEM_SHUTDOWN_EVENT				26
#define RTS_REDUNDANT_CONTROLLER_TRANSMIT_MESSAGE_EVENT		27
#define RTS_REDUNDANT_CONTROLLER_RECEIVE_MESSAGE_EVENT		28
#define RTS_REDUNDANT_CONTROLLER_STATUS_CHANGE_EVENT		29
#define RTS_REDUNDANT_CONTROLLER_PARITY_ERROR_EVENT		30
#define RTS_REDUNDANT_CONTROLLER_PARITY_ERROR_ACTION_EVENT	31
#define RTS_FORMAT_STARTED_EVENT				32
#define RTS_FORMAT_COMPLETED_EVENT				33
#define RTS_DRIVE_VALIDATION_EVENT				34
#define RTS_RESERVED_EVENT 					255

/* Event Log Information */
struct rts_event_log_information
{
    Word oldest_event_sequence_number;
    Word newest_event_sequence_number;
    Byte status;
};



/* Event RAID Set Information */
struct rts_event_raid_set_information
{
    Byte raid_set;
    Byte raid_level;
    Byte raid_set_status;
    Byte redundancy_group;
    Byte member;
    Byte number_of_members;
    Byte raid_set_partitions;
    Byte number_of_redundancy_groups;
};

struct rts_unknown_event
{
    Byte unknown_event_value;
    Byte pad[2];
};

struct rts_scsi_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte pad_a[2];
    Quad scdrp;
    Quad cdrp;
    Byte scsi_command[12];
    Byte phase_history[16];
    Byte request_sense_data[18];
    Byte retry_number;
    Byte pad_b;
};

struct rts_port_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte retry_number;
    Byte port_status;
    Quad scdrp;
    Quad cdrp;
    Byte scsi_command[12];
    Byte phase_history[16];
};

struct rts_parity_error_event
{
    Quad memory_address;
};

struct rts_create_started_event
{
    struct rts_event_raid_set_information raid_set_information;
    struct
    {
	Byte channel;
	Byte scsi_id;
    } member[1];
};

struct rts_rebuild_started_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte pad[2];
};

struct rts_create_completed_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte status;
    Byte pad;
};

struct rts_rebuild_completed_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte status;
    Byte pad;
};    

struct rts_dropped_drive_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte reason;
    Byte pad;
};    

struct rts_temperature_changed_event
{
    Byte temperature;
    Byte pad[3];
};

struct rts_bad_fan_event
{
    Byte fan;
    Byte pad[3];
};

struct rts_drive_seen_event
{
    Byte channel;
    Byte scsi_id;
    Byte pad[2];
};

struct rts_drive_gone_event
{
    Byte channel;
    Byte scsi_id;
    Byte pad[2];
};

struct rts_validation_conflict_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte data[225]; /* Complex structure */
};

struct rts_scsi_host_event
{
    Quad event_type;  /* Should this really be Quad and not Byte? */
    Quad cdrp;
    Byte target_id;
    Byte lun;
    Byte initiator_id;
    Byte htcb_flags;
    Word cdrp_host_flags;
    Word cdrp_flags;
    Byte error_type;
    Byte tag_type;
    Byte tag_number;
    Byte scsi_chip_status_register;
    Byte scsi_chip_interrupt_register;
    Byte scsi_status;
    Byte scsi_command[16];
    Byte request_sense[18];
};

struct rts_reassign_block_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte status;
    Byte pad;
    Quad scdrp;
    Quad cdrp;
    Quad reassigned_vbn;
    Quad reassigned_pbn;
};

struct rts_internal_env_system_event
{
    Byte type_of_device;
    Byte actual_device;
    Byte device_status;
    Byte pad_a;
    Word value;
    Byte pad_b[2];
};
    
struct rts_port_sync_negotiation_event
{
    Byte channel;
    Byte scsi_id;
    Word requested_sync_time_period;
    Word received_sync_time_period;
    Byte requested_sync_offset;
    Byte received_sync_offset;
};

struct rts_port_wide_negotiation_event
{
    Byte channel;
    Byte scsi_id;
    Byte requested_dma_width;
    Byte received_dma_width;
};

struct rts_host_sync_negotiation_event
{
    Byte host_initiator_id;
    Byte host_lun;
    Word requested_sync_time_period;
    Word returned_sync_time_period;
    Byte requested_sync_offset;
    Byte returned_sync_offset;
};

struct rts_host_wide_negotiation_event
{
    Byte host_initiator_id;
    Byte host_lun;
    Byte requested_dma_width;
    Byte returned_dma_width;
};

struct rts_forced_error_written_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte status;
    Byte pad;
    Quad scdrp;
    Quad cdrp;
    Quad forced_error_vbn;
    Quad forced_error_pbn;
};

struct rts_unexpected_port_interrupt_event
{
    Byte channel;
    Byte scsi_id;
    Byte scsi_chip_status_register;
    Byte scsi_chip_interrupt_register;
    Quad scpdt;
    Byte last_scsi_bus_phase;
    Byte current_scsi_bus_phase;
    Byte pad[2];
    Word port_flags;
    Word port_status;
    Byte message_out_data_buffer[12];
    Byte message_in_data_buffer[12];
};

struct rts_add_spare_started_event
{
    Byte channel;
    Byte scsi_id;
    Byte type;
    Byte pad;
};

struct rts_add_spare_completed_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte status;
    Byte pad;
};

struct rts_system_power_up_event
{
    struct rts_date_and_time time_stamp;
    Byte pad[2];
};

struct rts_system_shutdown_event
{
    Byte reason;
    Byte pad[3];
};

struct rts_redundant_controller_txrx_event
{
    Byte byte_count;
    Byte tag;
    Byte message_type;
    Byte reason_or_pad;
    Byte body[156];
    Byte channel;
    Byte pad[3];
};

struct rts_redundant_controller_status_change_event
{
    Byte status;
    Byte reason;
    Byte body[22];
    Byte pad[4];
};

struct rts_redundant_controller_parity_error_event
{
    Quad address;
};

struct rts_redundant_controller_parity_error_action_event
{
    Quad type;
    Quad where;
    Quad address;
    Quad action;
    Quad value;
};

struct rts_format_started_event
{
    Byte channel;
    Byte scsi_id;
    Byte pad[2];
};

struct rts_format_completed_event
{
    Byte channel;
    Byte scsi_id;
    Byte pad[2];
};

struct rts_drive_validation_event
{
    struct rts_event_raid_set_information raid_set_information;
    Byte channel;
    Byte scsi_id;
    Byte drive_status;
    Byte validation_status;
};



union rts_events
{
    struct rts_unknown_event			unknown;
    struct rts_scsi_event			scsi;
    struct rts_port_event			port;
    struct rts_parity_error_event		parity_error;
    struct rts_create_started_event		create_started;
    struct rts_rebuild_started_event		rebuild_started;
    struct rts_create_completed_event		create_completed;
    struct rts_rebuild_completed_event		rebuild_completed;
    struct rts_dropped_drive_event		dropped_drive;
    struct rts_temperature_changed_event    	temperature_changed;
    struct rts_bad_fan_event			bad_fan;
    struct rts_drive_seen_event			drive_seen;
    struct rts_drive_gone_event			drive_gone;
    struct rts_validation_conflict_event	validation_conflict;
    struct rts_scsi_host_event			scsi_host;
    struct rts_reassign_block_event		reassign_block;
    struct rts_internal_env_system_event 	internal_env_system;
    struct rts_port_sync_negotiation_event 	port_sync_negotiation;
    struct rts_port_wide_negotiation_event	port_wide_negotiation;
    struct rts_host_sync_negotiation_event 	host_sync_negotiation;
    struct rts_host_wide_negotiation_event	host_wide_negotiation;
    struct rts_forced_error_written_event	forced_error_written;
    struct rts_unexpected_port_interrupt_event	unexpected_port_interrupt;
    struct rts_add_spare_started_event		add_spare_started;
    struct rts_add_spare_completed_event	add_spare_completed;
    struct rts_system_power_up_event		system_power_up;
    struct rts_system_shutdown_event		system_shutdown;
    
    union
    {
	struct rts_redundant_controller_txrx_event		txrx;
	struct rts_redundant_controller_status_change_event status_change;
	struct rts_redundant_controller_parity_error_event 	parity_error;
	struct rts_redundant_controller_parity_error_action_event parity_error_action;
    } redundant_controller;
    
    struct rts_format_started_event		format_started;
    struct rts_format_completed_event		format_completed;
    struct rts_drive_validation_event		drive_validation;
    
    Byte dummy[233];
};




struct rts_event_information
{
    Byte event_log_version;
    Byte event_type;
    Word sequence_number;
    struct rts_date_and_time time_stamp;
    union rts_events data;
};



extern const char *
rts_strevtype(int event_type);

extern const char *
rts_strevent(struct rts_event_information *ep);

extern void
rts_print_event(FILE *fp,
		struct rts_event_information *ep);

#endif


