/*
** rts/print.c
**
** Viper RTS Print (of data structures) functions
**
** Copyright (c) 1998 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>

#include "rts/rts.h"
#include "rts/print.h"


void
rts_print_hex(FILE *fp,
	      Byte *bp,
	      int len)
{
    while (len-- > 0)
    {
	fprintf(fp, "%02X", *bp++);
	if (len > 0)
	    putc(' ', fp);
    }
}



void
rts_print_host_parameters(FILE *fp,
			  struct rts_host_parameters *hp)
{
    int i;

    
    fprintf(fp, "%-20s%d\n", "Channel", hp->channel);
    fprintf(fp, "%-20s%d\n", "SCSI ID", hp->scsi_id);
    fprintf(fp, "%-20s%s\n", "Tag Queuing", hp->tag_queuing ? "On" : "Off");
    fprintf(fp, "%-20s%s\n", "Sync Mode", hp->sync_mode ? "On" : "Off");
    fprintf(fp, "%-20s%d\n", "Sync Rate", hp->sync_rate);
    fprintf(fp, "%-20s%d\n", "Bus Width", hp->bus_width ? 16 : 8);
    fprintf(fp, "%-20s", "Termination");
    switch (hp->termination)
    {
      case 0:
	fputs("None\n", fp);
	break;
      case 1:
	fputs("Partial\n", fp);
	break;
      case 2:
	fputs("Full\n", fp);
	break;
      default:
	fputs("Unknown\n", fp);
    }
    fprintf(fp, "%-20s%02X\n", "Inquiry Peripheral",
	    hp->inquiry_peripheral);

    fprintf(fp, "LUN/Redundancy Group mapping:\n");
    
    for (i = 0; i < 32; i++)
	if (hp->host_lun_map[i] != i)
	    break;

    if (i == 32)
	fprintf(fp, "\tAll LUNs (32) map directly to redundancy groups\n");
    else
    {
	for (i = 0; i < 32; i++)
	    if (hp->host_lun_map[i] != 255)
		fprintf(fp, "\tLUN %2d -> Group %2d\n",
			i, hp->host_lun_map[i]);
    }
}


void
rts_print_redundancy_group_information(FILE *fp,
				       struct rts_redundancy_group_information *rp)
{
    time_t bt;
    int i;
     
    
    fprintf(fp, "Redundancy group: %d\n", rp->redundancy_group);
    fprintf(fp, "RAID set: %d\n", rp->raid_set);
    fprintf(fp, "RAID status: %s\n", rts_status2str(rp->raid_status));
    fprintf(fp, "RAID level: %s\n", rts_raid2str(rp->raid_level));
    if (rp->host_0_lun != 255)
	fprintf(fp, "Host 0 LUN: %d\n", rp->host_0_lun);
    if (rp->host_1_lun != 255)
	fprintf(fp, "Host 1 LUN: %d\n", rp->host_1_lun);
    if (rp->host_2_lun != 255)
	fprintf(fp, "Host 2 LUN: %d\n", rp->host_2_lun);
    if (rp->host_3_lun != 255)
	fprintf(fp, "Host 3 LUN: %d\n", rp->host_3_lun);
    bt = rts_mktime(&rp->time_stamp);
    fprintf(fp, "Created: %s", ctime(&bt));
    fprintf(fp, "Chunk size: %d blocks\n", rts_w2i(rp->chunk_size));
    fprintf(fp, "Capacity: %d blocks (%d MB)\n", rts_q2i(rp->capacity),
	    rts_q2i(rp->capacity)/2/1024);
    
    fprintf(fp, "Members: %d\n", rp->number_of_members);
    for (i = 0; i < rp->number_of_members; i++)
    {
	fprintf(fp, "Member %2d: Channel %d, SCSI ID %d\n",
		i,
		rp->member_descriptor[i].channel,
		rp->member_descriptor[i].scsi_id);
    }
}

