/*
** rts/rdwr.c
**
** Viper RTS Read & Write functions
**
** Copyright (c) 1998 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include "rts/rts.h"


int
rts_write_host_parameters(RTS *rp,
			  struct rts_host_parameters *hpp)
{
    int code;

    code = rts_write(rp, RTS_HOST_PARAMETERS, hpp, sizeof(*hpp));
    return code;
}


int
rts_write_system_parameters(RTS *rp,
			    struct rts_system_parameters *spp)
{
    int code;

    code = rts_write(rp, RTS_SYSTEM_PARAMETERS, spp, sizeof(*spp));
    return code;
}


int
rts_write_vendor_parameters(RTS *rp,
			    struct rts_vendor_parameters *vpp)
{
    int code;

    code = rts_write(rp, RTS_VENDOR_PARAMETERS, vpp, sizeof(*vpp));
    return code;
}


int
rts_write_mode_parameters(RTS *rp,
			  struct rts_mode_parameters *mpp)
{
    int code;

    code = rts_write(rp, RTS_MODE_PARAMETERS, mpp, sizeof(*mpp));
    return code;
}


int
rts_write_date_and_time(RTS *rp,
			time_t bt)
{
    struct rts_date_and_time datb;
    int code;


    if (rts_localtime(bt, &datb) == NULL)
	return -1;

    code = rts_write(rp, RTS_DATE_AND_TIME, &datb, sizeof(datb));
    return code;
}


int
rts_write_disk_parameters(RTS *rp,
			  struct rts_disk_parameters *dpp)
{
    int code;

    code = rts_write(rp, RTS_DISK_PARAMETERS, dpp, sizeof(*dpp));
    return code;
}



int
rts_write_redundant_controller_parameters(RTS *rp,
			  struct rts_redundant_controller_parameters *rcpp)
{
    int code;

    code = rts_write(rp, RTS_REDUNDANT_CONTROLLER_PARAMETERS,
		     rcpp, sizeof(*rcpp));
    return code;
}


int
rts_write_cache_parameters(RTS *rp,
			   struct rts_cache_parameters *cpp)
{
    int code;

    code = rts_write(rp, RTS_CACHE_PARAMETERS, cpp, sizeof(*cpp));
    return code;
}




int
rts_read_host_parameters(RTS *rp,
			 int channel,
			 struct rts_host_parameters *hpp)
{
    struct rts_read_host_parameters rhpb;
    int code;

    memset(&rhpb, 0, sizeof(rhpb));
    rhpb.channel = channel;

    code = rts_read(rp, RTS_HOST_PARAMETERS,
		    &rhpb, sizeof(rhpb), hpp, sizeof(*hpp));

    return code;
}


int
rts_read_system_parameters(RTS *rp,
			   struct rts_system_parameters *spp)
{
    int code;


    code = rts_read(rp, RTS_SYSTEM_PARAMETERS, NULL, 0, spp, sizeof(*spp));

    return code;
}


int
rts_read_vendor_parameters(RTS *rp,
			   struct rts_vendor_parameters *vpp)
{
    int code;


    code = rts_read(rp, RTS_VENDOR_PARAMETERS, NULL, 0, vpp, sizeof(*vpp));

    return code;
}


int
rts_read_mode_parameters(RTS *rp,
			 int channel,
			 struct rts_mode_parameters *mpp)
{
    struct rts_read_mode_parameters rmpb;
    int code;

    memset(&rmpb, 0, sizeof(rmpb));
    rmpb.channel = channel;

    code = rts_read(rp, RTS_MODE_PARAMETERS,
		    &rmpb, sizeof(rmpb), mpp, sizeof(*mpp));

    return code;
}


int
rts_read_date_and_time(RTS *rp,
		       time_t *bt)
{
    struct rts_date_and_time datb;
    int code;
    time_t t;


    code = rts_read(rp, RTS_DATE_AND_TIME, NULL, 0, &datb, sizeof(datb));
    if (code != 0)
	return code;

    t = rts_mktime(&datb);
    if (t == (time_t) -1)
	return -1;
    
    *bt = t;
    return 0;
}



int
rts_read_disk_parameters(RTS *rp,
			 struct rts_disk_parameters *dpp)
{
    int code;

    code = rts_read(rp, RTS_DISK_PARAMETERS, NULL, 0, dpp, sizeof(*dpp));

    return code;
}



int
rts_read_redundant_controller_parameters(RTS *rp,
			 struct rts_redundant_controller_parameters *rcpp)
{
    int code;

    code = rts_read(rp, RTS_REDUNDANT_CONTROLLER_PARAMETERS,
		    NULL, 0, rcpp, sizeof(*rcpp));

    return code;
}


int
rts_read_cache_parameters(RTS *rp,
			  struct rts_cache_parameters *cpp)
{
    int code;

    code = rts_read(rp, RTS_CACHE_PARAMETERS, NULL, 0, cpp, sizeof(*cpp));

    return code;
}



int
rts_read_static_system_information(RTS *rp,
				   struct rts_static_system_information *sp)
{
    int code;


    code = rts_read(rp, RTS_STATIC_SYSTEM_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}



int
rts_read_dynamic_system_information(RTS *rp,
				    struct rts_dynamic_system_information *sp)
{
    int code;


    code = rts_read(rp, RTS_DYNAMIC_SYSTEM_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}


int
rts_read_redundancy_group_information(RTS *rp,
				      int redundancy_group,
				      struct rts_redundancy_group_information *sp)
{
    int code;
    struct rts_read_redundancy_group_information rb;

    rb.redundancy_group = redundancy_group;

    code = rts_read(rp, RTS_REDUNDANCY_GROUP_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}



int
rts_read_environmental_information(RTS *rp,
				   struct rts_environmental_information *sp)
{
    int code;


    code = rts_read(rp, RTS_ENVIRONMENTAL_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}



int
rts_read_disk_drive_information(RTS *rp,
				int channel,
				int scsi_id,
				struct rts_disk_drive_information *sp)
{
    int code;
    struct rts_read_disk_drive_information rb;

    
    rb.channel = channel;
    rb.scsi_id = scsi_id;

    code = rts_read(rp, RTS_DISK_DRIVE_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}


int
rts_read_event_log_information(RTS *rp,
			       struct rts_event_log_information *sp)
{
    int code;


    code = rts_read(rp, RTS_EVENT_LOG_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}

int
rts_read_event_information(RTS *rp,
			   int event_sequence_number,
			   struct rts_event_information *sp)
{
    int code;
    struct rts_read_event_information rb;

    rb.event_sequence_number = rts_i2w(event_sequence_number);

    code = rts_read(rp, RTS_EVENT_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}


int
rts_read_create_information(RTS *rp,
			   int raid_set_number,
			   struct rts_create_information *sp)
{
    int code;
    struct rts_read_create_information rb;


    rb.raid_set_number = raid_set_number;

    code = rts_read(rp, RTS_CREATE_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}


int
rts_read_rebuild_information(RTS *rp,
			     int channel,
			     int scsi_id,
			     struct rts_rebuild_information *sp)
{
    int code;
    struct rts_read_rebuild_information rb;


    rb.channel = channel;
    rb.scsi_id = scsi_id;

    code = rts_read(rp, RTS_REBUILD_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}


int
rts_read_shutdown_information(RTS *rp,
			     struct rts_shutdown_information *sp)
{
    int code;

    
    code = rts_read(rp, RTS_SHUTDOWN_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}



int
rts_read_raid_set_information(RTS *rp,
			      int raid_set,
			      struct rts_raid_set_information *sp)
{
    int code;
    struct rts_read_raid_set_information rb;


    rb.raid_set = raid_set;

    code = rts_read(rp, RTS_RAID_SET_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}



int
rts_read_redundant_controller_information(RTS *rp,
		      struct rts_redundant_controller_information *sp)
{
    int code;


    code = rts_read(rp, RTS_REDUNDANT_CONTROLLER_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}


int
rts_read_io_statistical_information(RTS *rp,
				    unsigned long redundancy_groups,
				    int sample_period,
				    struct rts_io_statistical_information *sp)
{
    int code;
    struct rts_read_io_statistical_information rb;


    rb.redundancy_groups = rts_i2q(redundancy_groups);
    rb.sample_period = sample_period;

    code = rts_read(rp, RTS_IO_STATISTICAL_INFORMATION,
		    &rb, sizeof(rb), sp, sizeof(*sp));
    
    return code;
}


int
rts_read_session_information(RTS *rp,
			     struct rts_session_information *sp)
{
    int code;

    code = rts_read(rp, RTS_SESSION_INFORMATION,
		    NULL, 0, sp, sizeof(*sp));
    
    return code;
}

