/*
** scsi.c
**
** Sun user-level SCSI interface
**
** Copyright (c) 1998 Peter Eriksson <pen@lysator.liu.se>
*/

#include <unistd.h>

#include "scsi.h"


/* Generic SCSI CDB for SEND/RECEIVE DIAGNOSTIC */

struct scsi_diagnostic_cdb
{
    unsigned char command;
    unsigned char flags;
    unsigned char reserved;
    unsigned char msb_parlen;
    unsigned char lsb_parlen;
    unsigned char control;
};

/* Flags consists of the following bits:

   Bit	Meaning
   ---- --------
   7-5	LUN
   4	Page Format 	(should be Zero)
   3	Reserved    	(should be Zero)
   2	SelfTest    	(should be Zero)
   1    Device-Offline	(should be Zero)
   0	Unit-Offline    (should be Zero)
*/


int
scsi_command(int fd,
	     int cmd,
	     int flags, 
	     void *bufp,
	     int buflen,
	     int timeout)
{
    struct uscsi_cmd usbuf;
    struct scsi_diagnostic_cdb cdb;

    
    /* Set up the SCSI CDB */
    memset(&cdb, 0, sizeof(cdb));
    cdb.command = cmd;
    if (bufp)
    {
	cdb.msb_parlen = (buflen >> 8) & 0xFF;
	cdb.lsb_parlen = buflen & 0xFF;
    }
	
    /* Set up the USCSI buffer */
    memset(&usbuf, 0, sizeof(usbuf));
    usbuf.uscsi_flags = flags | USCSI_DIAGNOSE;
    usbuf.uscsi_timeout = timeout;
    usbuf.uscsi_cdb = (caddr_t) &cdb;
    usbuf.uscsi_cdblen = sizeof(cdb);
    usbuf.uscsi_bufaddr = (caddr_t) bufp;
    usbuf.uscsi_buflen = buflen;

    if (ioctl(fd, USCSICMD, &usbuf) < 0)
	return -1;

    return (usbuf.uscsi_status >> 1) & 31;
}

